<?php

namespace app\api\controller;

use app\admin\model\school\classes\VirtualHead;
use app\common\controller\Api;
use app\common\exception\UploadException;
use app\common\library\Upload;
use app\common\library\Virtual;
use app\common\model\Area;
use app\common\model\Version;
use fast\Random;
use think\captcha\Captcha;
use think\Config;
use think\Hook;

/**
 * 公共接口
 */
class Common extends Api
{
    protected $noNeedLogin = ['init', 'captcha','virtualgenerate','get_week_by_time'];
    protected $noNeedRight = '*';

    public function _initialize()
    {

        if (isset($_SERVER['HTTP_ORIGIN'])) {
            header('Access-Control-Expose-Headers: __token__');//跨域让客户端获取到
        }
        //跨域检测
        check_cors_request();

        if (!isset($_COOKIE['PHPSESSID'])) {
            Config::set('session.id', $this->request->server("HTTP_SID"));
        }
        parent::_initialize();
    }

    /**
     * 加载初始化
     *
     * @ApiParams (name="version", type="string", required=true, description="版本号")
     * @ApiParams (name="lng", type="string", required=true, description="经度")
     * @ApiParams (name="lat", type="string", required=true, description="纬度")
     */
    public function init()
    {
        if ($version = $this->request->request('version')) {
            $lng = $this->request->request('lng');
            $lat = $this->request->request('lat');

            //配置信息
            $upload = Config::get('upload');
            //如果非服务端中转模式需要修改为中转
            if ($upload['storage'] != 'local' && isset($upload['uploadmode']) && $upload['uploadmode'] != 'server') {
                //临时修改上传模式为服务端中转
                set_addon_config($upload['storage'], ["uploadmode" => "server"], false);

                $upload = \app\common\model\Config::upload();
                // 上传信息配置后
                Hook::listen("upload_config_init", $upload);

                $upload = Config::set('upload', array_merge(Config::get('upload'), $upload));
            }

            $upload['cdnurl'] = $upload['cdnurl'] ? $upload['cdnurl'] : cdnurl('', true);
            $upload['uploadurl'] = preg_match("/^((?:[a-z]+:)?\/\/)(.*)/i", $upload['uploadurl']) ? $upload['uploadurl'] : url($upload['storage'] == 'local' ? '/api/common/upload' : $upload['uploadurl'], '', false, true);

            $content = [
                'citydata'    => Area::getCityFromLngLat($lng, $lat),
                'versiondata' => Version::check($version),
                'uploaddata'  => $upload,
                'coverdata'   => Config::get("cover"),
            ];
            $this->success('', $content);
        } else {
            $this->error(__('Invalid parameters'));
        }
    }

    /**
     * 用户上传文件
     * @ApiMethod (POST)
     * @ApiParams (name="file", type="File", required=true, description="文件流")
     * @ApiParams (name="category", type="string", required=true, description="分类标识：category1=非机密类1,category2=非机密类2,cert=证件机密类,code=二维码类,user=用户普通上传")
     *
     */
    public function upload()
    {
        Config::set('default_return_type', 'json');
        //必须设定cdnurl为空,否则cdnurl函数计算错误
        Config::set('upload.cdnurl', '');
        $category = $this->request->post("category",'user');
//        var_dump($category);die;
        $chunkid = $this->request->post("chunkid");
        if ($chunkid) {
            if (!Config::get('upload.chunking')) {
                $this->error(__('Chunk file disabled'));
            }
            $action = $this->request->post("action");
            $chunkindex = $this->request->post("chunkindex/d");
            $chunkcount = $this->request->post("chunkcount/d");
            $filename = $this->request->post("filename");

            $method = $this->request->method(true);
            if ($action == 'merge') {
                $attachment = null;
                //合并分片文件
                try {
                    $upload = new Upload();
                    $attachment = $upload->merge($chunkid, $chunkcount, $filename);
                } catch (UploadException $e) {
                    $this->error($e->getMessage());
                }
                $this->success(__('Uploaded successful'), ['url' => $attachment->url, 'fullurl' => cdnurl($attachment->url, true)]);
            } elseif ($method == 'clean') {
                //删除冗余的分片文件
                try {
                    $upload = new Upload();
                    $upload->clean($chunkid);
                } catch (UploadException $e) {
                    $this->error($e->getMessage());
                }
                $this->success();
            } else {
                //上传分片文件
                //默认普通上传文件
                $file = $this->request->file('file');
                try {

                    $upload = new Upload($file);
                    $upload->chunk($chunkid, $chunkindex, $chunkcount);

                } catch (UploadException $e) {
                    $this->error($e->getMessage());
                }
                $this->success();
            }
        } else {
            $attachment = null;
            //默认普通上传文件
            $file = $this->request->file('file');
            try {

                $upload = new Upload($file,$category);
                $attachment = $upload->upload();

//                $attachment = $upload->upload();
            } catch (UploadException $e) {
                $this->error($e->getMessage().$e->getFile().$e->getLine());
            } catch (\Exception $e) {
                $this->error($e->getMessage().$e->getFile().$e->getLine());
            }

            $this->success(__('Uploaded successful'), ['url' => $attachment->url, 'fullurl' => cdnurl($attachment->url, true)]);
        }

    }

    /**
     * 验证码
     * @ApiParams (name="id", type="string", required=true, description="要生成验证码的标识")
     * @return \think\Response
     */
    public function captcha($id = "")
    {
        \think\Config::set([
            'captcha' => array_merge(config('captcha'), [
                'fontSize' => 44,
                'imageH'   => 150,
                'imageW'   => 350,
            ])
        ]);
        $captcha = new Captcha((array)Config::get('captcha'));
        return $captcha->entry($id);




    }


    /**
     * @ApiTitle(生成随机昵称和头像)
     * @ApiSummary(生成随机昵称和头像)
     * @ApiRoute(/api/common/virtualgenerate)
     * @ApiMethod(GET)
     * @ApiParams(name = "number", type = "int",required=true,description = "生成数量")
     * @ApiReturn({ unpaid_user_data 参与中  paid_user_data 已报名 })
     */
    public function virtualgenerate(){
        $number = $this->request->get('number/d','');
        if(empty($number)){
            $this->error(__('缺少必要参数'));
        }
        try {
            $res =  (new Virtual)->getVirtualUser($number);
        } catch (\Exception $e){
//            Log::log($e->getMessage());
            $this->error($e->getMessage(),['errcode'=>$e->getCode()]);
        }
        $this->success('生成成功', $res);
    }



        /**
     * @ApiTitle(通过时间点得到当前的星期数据)
     * @ApiSummary(通过时间点得到当前的星期数据)
     * @ApiRoute(/api/common/get_week_by_time)
     * @ApiMethod(GET)
     * @ApiParams(name="time",type="string",required=true,description="选择的时间点")
     * @ApiReturn({ unpaid_user_data 参与中  paid_user_data 已报名 })
     */
    public function get_week_by_time(){
        $time = $this->request->get('time/s','');

//        if(empty($time)){
//            $this->error(__('缺少必要参数'));
//        }
        try {
            $res =  (new Virtual)->getWeekByTime($time);
        } catch (\Exception $e){
//            Log::log($e->getMessage());
            $this->error($e->getMessage(),['errcode'=>$e->getCode()]);
        }
        $this->success('生成成功', $res);
    }

}
